/*
*********************************************************************************************************
*                                       STMicroelectronics STR711
*                                         Board Support Package
*                 
*
*                                (c) Copyright 2005, Micrium, Weston, FL
*                                          All Rights Reserved
*
*
* File      : BSP.C
*
* Changes   : Michael Fischer
*             This port based on AN-1710. I have make some changes to 
*             support the CrossWorks for ARM toolchain from  Rowley Associates. 
*
* Toolchain : CrossWorks for ARM 1.5 Build 2 and higher
*********************************************************************************************************
*/

#define  BSP_GLOBALS
#include <includes.h>

/*
*********************************************************************************************************
*                                               CONSTANTS                                                
*********************************************************************************************************
*/

#define  OS_CPU_IRQ_ISR   irq_handler        /* os_cpu_a.s */
#define  OS_CPU_FIQ_ISR   fiq_handler        /* os_cpu_a.s */

#define  BSP_UNDEF_INSTRUCTION_VECTOR_ADDR   (*(INT32U *)        0x00000004L)
#define  BSP_SWI_VECTOR_ADDR                 (*(INT32U *)        0x00000008L)
#define  BSP_PREFETCH_ABORT_VECTOR_ADDR      (*(INT32U *)        0x0000000CL)
#define  BSP_DATA_ABORT_VECTOR_ADDR          (*(INT32U *)        0x00000010L)
#define  BSP_IRQ_VECTOR_ADDR                 (*(INT32U *)        0x00000018L)
#define  BSP_FIQ_VECTOR_ADDR                 (*(INT32U *)        0x0000001CL)

#define  BSP_IRQ_ISR_ADDR                    (*(INT32U *)        0x00000038L)
#define  BSP_FIQ_ISR_ADDR                    (*(INT32U *)        0x0000003CL)

#define  BSP_OSC_FREQ                                                4000000
#define  BSP_CPU_FREQ                                               32000000

#define  BSP_TMR0_PRESCALER                                             0x0F

#define  BSP_TMR1_PRESCALER                                             0x0F

#define  BSP_TMR0_INT                                                      0
#define  BSP_UART0_INT                                                     9

#define  BSP_NBR_INT                                                      32

#define  BSP_VECT_SET                                                      0
#define  BSP_VECT_ERR                                                      1

#define  LED_MASK                                                     0x00F0

/*
*********************************************************************************************************
*                                               DATA TYPES
*********************************************************************************************************
*/

typedef  void (*BSP_PFNCT)(void);

/*
*********************************************************************************************************
*                                                VARIABLES
*********************************************************************************************************
*/

static  BSP_PFNCT  BSP_ISR_Table[BSP_NBR_INT];

static  INT32U     BSP_Peripheral_Clk1_Freq;
static  INT32U     BSP_Peripheral_Clk2_Freq;

static  INT16U     BSP_Tmr0_Rst_Value;

/*
*********************************************************************************************************
*                                               PROTOTYPES
*********************************************************************************************************
*/

static  void  BSP_IO_Init(void);

static  void  Tmr_TickISR_Handler (void);
static  void  Tmr_TickInit(void);

static  void  EIC_Init(void);

/*
*********************************************************************************************************
*                                      Get the CPU Clock Frequency
*
* Description: This function determines MCLK, the frequency at which the CPU is running.
*
* Arguments  : none
*********************************************************************************************************
*/

INT32U  BSP_CPU_ClkFreq (void)
{
    INT32U  cpu_clk_freq;
    INT8U   mult;
    INT8U   pll_div;  
    INT8U   main_div;
    

    if (((PRCCU_CFR) & 0x00008000) != 0) {             /* Determine if the clock is being divided by 2 */
        cpu_clk_freq = BSP_OSC_FREQ / 2;
    } else {
        cpu_clk_freq = BSP_OSC_FREQ;
    }
                                                       /* Determine if the PLL is being used           */
    if ((((PRCCU_CFR) & 0x00000003) == 0x00000003) && (((PRCCU_PWRCR) & 0x1000) == 0x1000)) {
                                                       /* Multiply the clock by the appropriate factor */
        mult = (INT8U)(((PRCCU_PLL1CR) & 0x00000030) >> 4);
        
        switch (mult) {
            case 0:
                 cpu_clk_freq *= 20;
                 break;

            case 1:
                 cpu_clk_freq *= 12;
                 break;

            case 2:
                 cpu_clk_freq *= 24;
                 break;

            case 3:
                 cpu_clk_freq *= 16;
                 break;
        }                        
                                                      /* Divide the clock by the appropriate factor    */
        pll_div = (INT8U)((PRCCU_PLL1CR) & 0x00000007);
        
        if (pll_div != 7) {
            cpu_clk_freq /= (pll_div + 1);
        }
    }   
                                                     /* Again, divide the clock appropriately          */
    main_div      = (INT8U)(PRCCU_MDIVR & 0x0003);
    cpu_clk_freq /= (1 << main_div);

    return (cpu_clk_freq);
}

/*
*********************************************************************************************************
*                                   Get the Peripheral Clock Frequency
*
* Description: This function determines the clock frequency for APB1 and APB2, and sets variables 
*              corresponding to these values.
*
* Arguments  : none
*********************************************************************************************************
*/

static  void  BSP_Set_CPU_ClkFreqPeripheral (void)
{
    INT32U  cpu_peripheral_clk_freq;
    INT8U   mult;
    INT8U   pll_div;  
    INT8U   peripheral1_div;
    INT8U   peripheral2_div;
    

    if (((PRCCU_CFR) & 0x00008000) != 0) {             /* Determine if the clock is being divided by 2 */
        cpu_peripheral_clk_freq = BSP_OSC_FREQ / 2;
    } else {
        cpu_peripheral_clk_freq = BSP_OSC_FREQ;
    }
                                                       /* Determine if the PLL is being used           */
    if ((((PRCCU_CFR) & 0x00000003) == 0x00000003) && (((PRCCU_PWRCR) & 0x1000) == 0x1000)) {
                                                       /* Multiply the clock by the appropriate factor */
        mult = (INT8U)(((PRCCU_PLL1CR) & 0x00000030) >> 4);
        
        switch (mult) {
            case 0:
                 cpu_peripheral_clk_freq *= 20;
                 break;

            case 1:
                 cpu_peripheral_clk_freq *= 12;
                 break;

            case 2:
                 cpu_peripheral_clk_freq *= 24;
                 break;

            case 3:
                 cpu_peripheral_clk_freq *= 16;
                 break;
        } 
                                                      /* Divide the clock by the appropriate factor    */
        pll_div = (INT8U)((PRCCU_PLL1CR) & 0x00000007);
        
        if (pll_div != 7) {
            cpu_peripheral_clk_freq /= (pll_div + 1);
        }
    }   
                                                      /* Divide by the appropriate factor for APB1     */
    peripheral1_div          = (INT8U)(PRCCU_PDIVR & 0x0003);
    BSP_Peripheral_Clk1_Freq = cpu_peripheral_clk_freq / (1 << peripheral1_div);
                                                      /* Divide by the appropriate factor for APB2     */
    peripheral2_div          = (INT8U)((PRCCU_PDIVR >> 8) & 0x0003);
    BSP_Peripheral_Clk2_Freq = cpu_peripheral_clk_freq / (1 << peripheral2_div);
}

/*
*********************************************************************************************************
*                                         BSP INITIALIZATION
*
* Description : This function should be called by your application code before you make use of any of the
*               functions found in this module.
*
* Arguments   : none
*********************************************************************************************************
*/

void  BSP_Init (void)
{
    PRCCU_PLL1CR                      = 0x00000070;           /* Assumed we have a CK input frequency */
                                                              /* of 4 MHZ, the CPU frequency will set */
                                                              /* to 32 MHz                            */    
    
    PRCCU_CFR                        |= 0x00000003;

    BSP_IRQ_VECTOR_ADDR               = 0xE59FF018;             /* LDR PC,[PC,#0x18] instruction       */
    BSP_IRQ_ISR_ADDR                  = (INT32U)OS_CPU_IRQ_ISR; /* IRQ exception vector address        */

    BSP_FIQ_VECTOR_ADDR               = 0xE59FF018;             /* LDR PC,[PC,#0x18] instruction       */
    BSP_FIQ_ISR_ADDR                  = (INT32U)OS_CPU_FIQ_ISR; /* FIQ exception vector address        */

    BSP_UNDEF_INSTRUCTION_VECTOR_ADDR = 0xEAFFFFFE;             /* Jump to itself                      */
    BSP_SWI_VECTOR_ADDR               = 0xEAFFFFFE;
    BSP_PREFETCH_ABORT_VECTOR_ADDR    = 0xEAFFFFFE;
    BSP_DATA_ABORT_VECTOR_ADDR        = 0xEAFFFFFE;
    BSP_FIQ_VECTOR_ADDR               = 0xEAFFFFFE;


    BSP_Set_CPU_ClkFreqPeripheral();              /* Set variables reflecting the CPU's frequency      */
                                                  /* Wait until the new CPU frequency is in effect     */
    while (BSP_Peripheral_Clk1_Freq != BSP_CPU_FREQ) {
        BSP_Set_CPU_ClkFreqPeripheral();
    }

    BSP_IO_Init();                                /* Initialize the board's I/Os                       */
    
    EIC_Init();                                   /* Initialize the Enhanced Interrupt Controller      */

    LED_Init();                                   /* Initialize the I/Os for the LED controls          */

    Tmr_TickInit();                               /* Initialize the uC/OS-II tick interrupt            */    
}

/*
*********************************************************************************************************
*                                       SET INTERRUPT VECTOR
*
* Description : This function sets the Interrut Vector Register for the device to the device's
*               interrupt channel number.  This number is then used as an index into a table of interrupt 
*               handlers.
*
* Arguments   : device    corresponds to the device's interrupt channel number
*               handler   the function to be used as the device's interrupt handler
*
* Returns     : 0 if the handler was added to the table, 1 otherwise
*********************************************************************************************************
*/

static  INT8U  BSP_VectSet (INT16U device, BSP_PFNCT handler)
{
#if OS_CRITICAL_METHOD == 3                  /* Allocate storage for CPU status register               */
    OS_CPU_SR  cpu_sr = 0;
#endif    

    if (device < BSP_NBR_INT) {              /* Make sure there is a space for the device in the table */
                                             /* Set the lower 16-bits of the device's IVR              */
                                             
        EIC_SIRn[device]      = ((INT32U)device) << 16; 
        
        OS_ENTER_CRITICAL();
        BSP_ISR_Table[device] = handler;     /* Add the device's handler to the table                  */
        OS_EXIT_CRITICAL();
        
        return BSP_VECT_SET;
    } else {
        return BSP_VECT_ERR;
    }
}    

/*
*********************************************************************************************************
*                                    INITIALIZE INTERRUPT CONTROLLER
*
* Description : This function enables interrupts through the Enhanced Interrupt Controller (EIC). This
*               function also intializes the table that is used to determine the appropriate interrupt
*               handler for each interrupting device.
*
* Arguments   : none
*********************************************************************************************************
*/

static  void  EIC_Init (void)
{
    INT16U  i;
    
    EIC_ICR = 0x00000001;		                                         /* Enable interrupts (IRQ)        */
    
    for (i = 0; i < BSP_NBR_INT; i++) {                              /* Insert null interrupt handlers */
        BSP_VectSet(i, (BSP_PFNCT)0);
    } 
}

/*
*********************************************************************************************************
*                                      DISABLE ALL INTERRUPTS
*
* Description : This function disables all interrupts from the interrupt controller.
*               
* Arguments   : none
*********************************************************************************************************
*/

void  BSP_IntDisAll (void)
{
    EIC_ICR = 0x00000000;                                          /* Disable ALL interrupts          */
}

/*
*********************************************************************************************************
*                                           IRQ ISR HANDLER
*
* Description : This function is called by OS_CPU_IRQ_ISR() to determine the source of the interrupt
*               and process it accordingly.
*
* Arguments   : none
*********************************************************************************************************
*/

void  OS_CPU_IRQ_ISR_Handler (void)
{
    INT16U     idx;
    BSP_PFNCT  pfnct;
    
   
    idx   = (INT16U)((EIC_IVR) & 0x0000FFFF);  /* Get the index corresponding to the device's handler */
    pfnct = BSP_ISR_Table[idx];                 /* Read the interrupt vector from the table            */
    if (pfnct != (BSP_PFNCT)0) {                /* Make sure we don't have a NULL pointer              */
        (*pfnct)();                             /* Execute the ISR for the interrupting device         */
    }
}

/*
*********************************************************************************************************
*                                           FIQ ISR HANDLER
*
* Description : This function is called by OS_CPU_FIQ_ISR() to determine the source of the interrupt
*               and process it accordingly.
*
* Arguments   : none
*********************************************************************************************************
*/

void  OS_CPU_FIQ_ISR_Handler (void)
{
}

/*
*********************************************************************************************************
*                                          INITIALIZE I/Os
*
* Description : This function initializes the GPIO pins.  All the I/O pins are initialized in this 
*               function so you don't have to look at multiple places for I/O initialization.
*
*               For more information how to configure the ports, take a look in the 
*               STR71x MICROCONTROLLER FAMILIY - REFERENCE MANUAL (p.67, Table 20, Rev.6 March 2005)  
*
* Arguments   : none
*********************************************************************************************************
*/

static  void  BSP_IO_Init (void)
{
    IOPORT0_PC0 = 0xFFFF;                         /* Pins 8 and 9 in Port 0 correspond to UART 0        */
    IOPORT0_PC1 = 0x0300;
    IOPORT0_PC2 = 0xFFFF;

    /* LEDs */
    IOPORT0_PC0 |= LED_MASK;                     /* Pins 4,5,6 and 7 in Port 0 correspond to the LEDs   */
    IOPORT0_PC1 &= ~LED_MASK;
    IOPORT0_PC2 |= LED_MASK;
    IOPORT0_PD  |= LED_MASK;

    /* Push buttons P0.15 */   
    IOPORT0_PC0 |= 0x8000;
    IOPORT0_PC1 &= ~0x8000;
    IOPORT0_PC2 &= ~0x8000;
    
    /* Push buttons P1.9 */   
    IOPORT1_PC0 |= 0x0200;
    IOPORT1_PC1 &= ~0x0200;
    IOPORT1_PC2 &= ~0x0200;
}

/*
*********************************************************************************************************
*                                         GET 'PUSH BUTTON' STATUS
*
* Description : This function is used to get the status of any push button on the board.
*
* Arguments   : push_button    is the number of the push button to probe
*                              1    probe the "WAKE-UP" push button
*                              2    probe the "BUT" push button 
*********************************************************************************************************
*/

BOOLEAN  PB_GetStatus (INT8U push_button_id)
{
    BOOLEAN  status;

    status = FALSE;
    switch (push_button_id) {
        case 1:
             if (((IOPORT0_PD) & (1 << 15)) != 0) {
                 return (TRUE);
             }
             break;

        case 2:
             if (((IOPORT1_PD) & (1 << 9)) != 0) {
                 return (TRUE);
             }
             break;

        default:
             break;
    }
    return (status);
}

/*
*********************************************************************************************************
*                                           LED INITIALIZATION
*
* Description : This function initializes the board's LEDs.
* The LEDs are connected to P0.4 to P0.7.
*
* Arguments   : none
*********************************************************************************************************
*/

void  LED_Init (void)
{
    LED_Off(0);
}

/*
*********************************************************************************************************
*                                                 LED ON
*
* Description : This function is used to control any or all the LEDs on the board.
*
* Arguments   : led    is the number of the LED to control
*                      0    indicates that you want to turn on all of the board's LEDs
*********************************************************************************************************
*/

void  LED_On (INT8U led)
{
    switch (led) {
        case 0:
             IOPORT0_PD &= ~LED_MASK;
             break;

        case 1:
             IOPORT0_PD &= ~0x0040;
             break;

        case 2:
             IOPORT0_PD &= ~0x0010;
             break;
             
        case 3:
             IOPORT0_PD &= ~0x0020;
             break;
         
        case 4:
             IOPORT0_PD &= ~0x0080;
             break;
    } 
}

/*
*********************************************************************************************************
*                                                LED OFF
*
* Description : This function is used to control any or all the LEDs on the board.
*
* Arguments   : led    is the number of the LED to turn OFF
*                      0    indicates that you want to turn off all of the board's LEDs
*********************************************************************************************************
*/

void  LED_Off (INT8U led)
{
    switch (led) {
        case 0:
             IOPORT0_PD |= LED_MASK;
             break;

        case 1:
             IOPORT0_PD |= 0x0040;
             break;

        case 2:
             IOPORT0_PD |= 0x0010;
             break;
             
        case 3:
             IOPORT0_PD |= 0x0020;
             break;
         
        case 4:
             IOPORT0_PD |= 0x0080;
             break;        
    }
}

/*
*********************************************************************************************************
*                                              LED TOGGLE
*
* Description : This function is used to alternate the state of an LED
*
* Arguments   : led    is the number of the LED to control
*                      0    indicates that you want ALL the LEDs to toggle
*********************************************************************************************************
*/

void  LED_Toggle (INT8U led)
{
    switch (led) {
        case 0:                                                                 
             IOPORT0_PD ^= LED_MASK;
             break;

        case 1:                                                                 
             IOPORT0_PD ^= 0x0040;
             break;

        case 2:                                                                    
             IOPORT0_PD ^= 0x0010;
             break;
             
        case 3:                                                                   
             IOPORT0_PD ^= 0x0020;
             break;
         
        case 4:
             IOPORT0_PD ^= 0x0080;
             break;        
    }
}


/*
*********************************************************************************************************
*                                         TICKER INITIALIZATION
*
* Description : This function is called to initialize uC/OS-II's tick source (typically a timer 
*               generating interrupts every 1 to 100 mS).
*
*               We decided to use Timer #0 as the tick interrupt source.
*
* Arguments   : none
*********************************************************************************************************
*/

static  void  Tmr_TickInit(void)
{
    INT8U  err;
                                             
                                                 /* Calculate the reset value for Timer 0             */
    BSP_Tmr0_Rst_Value = (BSP_Peripheral_Clk2_Freq / (BSP_TMR0_PRESCALER + 1)) / OS_TICKS_PER_SEC;
                                                 /* Set up the IVR                                    */                                                                      
    err                = BSP_VectSet((INT16U)BSP_TMR0_INT, (BSP_PFNCT)Tmr_TickISR_Handler);    

    if (err == BSP_VECT_SET) {                           
        EIC_SIRn[BSP_TMR0_INT] |= 0x00000001;                       /* Set the priority for Timer 0   */  
    }
    EIC_IER   = 1 << BSP_TMR0_INT;                                  /* Enable the interrupt           */

    TIM0_CR2  = 0x4000 | BSP_TMR0_PRESCALER;    /* Set up the Timer 0 interrupt                       */
    TIM0_CR1  = 0x8040;                         /* Enable the timer in Output Compare mode            */        
    TIM0_OCAR = BSP_Tmr0_Rst_Value;     	      /* Set the timer's period                             */
    TIM0_CNTR = 0xFFF0;                         /* Reset the counter                                  */
}

/*
*********************************************************************************************************
*                                            TICK IRQ HANDLER
*
* Description : This function handles the timer interrupt that is used to generate TICKs for uC/OS-II.
*
* Arguments   : none
*
* Note(s)     : The TICK ISR uses Timer 0
*********************************************************************************************************
*/

static void  Tmr_TickISR_Handler (void)
{
    TIM0_SR  &= ~0x4000;	                      /* Clear the timer's interrupt pending bit             */
    EIC_IPR   = 0x00000001;                     /* Clear the timer's interrupt pending bit in the EIC  */
    TIM0_OCAR = BSP_Tmr0_Rst_Value;	            /* Reset the timer's period                            */
    TIM0_CNTR = 0xFFF0;                         /* Start the timer again                               */

    OSTimeTick();                               /* Call uC/OS-II's OSTimeTick() to signal a tick       */
}

/*
*********************************************************************************************************
*                          READ TIMER USED TO MEASURE INTERRUPT DISABLE TIME
*
* Description : This function is called to read the current counts of the interrupt disable time 
*               measurement timer.  Timer 1 is used for this measurement.
*
* Arguments   : none
*
* Returns     : The 16 bit counts of the timer assuming the timer is an UP counter.
*********************************************************************************************************
*/

#if OS_CPU_INT_DIS_MEAS_EN > 0
INT16U  OS_CPU_IntDisMeasTmrRd (void)
{
    INT16U  cnts;


    cnts = TIM1_CNTR;
    return (cnts);
}
#endif

/*
*********************************************************************************************************
*                                     INITIALIZE TIMER FOR uC/OS-View
*
* Description : This function is called to by uC/OS-View to initialize the free running timer that is
*               used to make time measurements.
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/

#if OS_VIEW_MODULE > 0
void  OSView_TmrInit (void)
{
    TIM1_CR2  = 0x0000 | BSP_TMR1_PRESCALER;    /* Set up the timer to run continuously               */
    TIM1_CR1  = 0x8000;                         /* Start the timer                                    */
}
#endif

/*
*********************************************************************************************************
*                                     READ TIMER FOR uC/OS-View
*
* Description : This function is called to read the current counts of a 16-bit free running timer.
*
* Arguments   : none
*
* Returns     : A 32-bit value representing the current time
*********************************************************************************************************
*/

#if OS_VIEW_MODULE > 0
INT32U  OSView_TmrRd (void)
{
    INT32U  cycles;


    cycles = (INT32U)(TIM1_CNTR);                         /* Get the current value of Timer 1         */
    return (cycles);
}
#endif

/*
*********************************************************************************************************
*                                           UART INITIALIZATION
*
* Description : This function is called by uC/OS-View to initialize the UART.
*
* Arguments   : baud_rate    The baud rate to which the UART will be initialized
*********************************************************************************************************
*/

#if OS_VIEW_MODULE > 0
void  OSView_UARTInit (INT32U baud_rate)
{
    INT8U  err;


    err                                = BSP_VectSet((INT16U)BSP_UART0_INT, (BSP_PFNCT)UART_ISR_Handler);
    
    if (err == BSP_VECT_SET) {
        EIC_SIRn[BSP_UART0_INT]       |= 0x00000003;                 /* Set the priority for UART 0    */      
    }
    EIC_IER                           |= 1 << BSP_UART0_INT;         /* Enable the interrupt           */

                                                                     /* Set the UART's baud rate       */
    UART0_BR  = (INT16U)(BSP_Peripheral_Clk1_Freq / (16 * baud_rate));  
    UART0_TOR = 0x00FF;                                              /* Set the UART's timeout register*/
    UART0_GTR = 0x0080;                                              /* Set the UART's Guard time      */
    UART0_IER = 0x0083;                                              /* Enable RX and TX interrputs    */
    UART0_CR  = 0x0189;                                              /* Start the UART                 */
}
#endif

/*
*********************************************************************************************************
*                                            UART IRQ HANDLER
*
* Description : This function handles the UART interrupt by calling the uC/OS-View handler
*
* Arguments   : none
*********************************************************************************************************
*/
#if OS_VIEW_MODULE > 0
void  UART_ISR_Handler (void)
{
    OSView_RxTxISRHandler();
    EIC_IPR  = 0x00000200;                       /* Clear the UART's bit in the EIC                    */
}
#endif


/*** EOF ***/



